package com.clx.performance.service.impl.collect;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.clx.performance.dao.collect.CollectDriverTransportIntentionDao;
import com.clx.performance.dao.collect.CollectDriverTransportIntentionRecordDao;
import com.clx.performance.enums.collect.CollectDriverTransportIntentionEnum;
import com.clx.performance.model.collect.CollectDriverTransportIntention;
import com.clx.performance.model.collect.CollectDriverTransportIntentionRecord;
import com.clx.performance.param.app.collect.AppCarrierDriverCollectTransportIntentionParam;
import com.clx.performance.param.app.collect.CarrierDriverCollectTransportIntentionListParam;
import com.clx.performance.param.app.collect.CarrierDriverCollectTransportIntentionParam;
import com.clx.performance.param.app.collect.CarrierDriverCollectTransportIntentionRecordParam;
import com.clx.performance.service.collect.CollectDriverTransportIntentionService;
import com.clx.performance.struct.collect.CollectDriverTransportIntentionDaoStruct;
import com.clx.performance.struct.collect.CollectDriverTransportIntentionRecordDaoStruct;
import com.clx.performance.vo.pc.collect.CarrierDriverCollectTransportIntentionRecordVO;
import com.clx.performance.vo.pc.collect.CarrierDriverCollectTransportIntentionVO;
import com.msl.user.utils.TokenUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author kavin
 * Date 2024-10-17
 * Time 16:26
 */
@Service
@Slf4j
@AllArgsConstructor
public class CollectDriverTransportIntentionServiceImpl implements CollectDriverTransportIntentionService {

    private final CollectDriverTransportIntentionDao collectDriverTransportIntentionDao;
    private final CollectDriverTransportIntentionRecordDao collectDriverTransportIntentionRecordDao;
    private final CollectDriverTransportIntentionDaoStruct collectDriverTransportIntentionDaoStruct;
    private final CollectDriverTransportIntentionRecordDaoStruct collectDriverTransportIntentionRecordDaoStruct;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer saveCarrierDriverCollectTransportIntention(CarrierDriverCollectTransportIntentionParam param) {

        if (Objects.isNull(param) || param.getCollectIntentionRecordList().isEmpty()) {
            // Todo 提示信息
            return 0;
        }

        CollectDriverTransportIntention collectDriverTransportIntention = new CollectDriverTransportIntention();
        // 默认排序
        collectDriverTransportIntention.setSortNo(1);
        collectDriverTransportIntention.setCollectName(param.getCollectName());
        collectDriverTransportIntention.setCollectDescribe(param.getCollectDescribe());
        collectDriverTransportIntention.setCollectCheckType(param.getCollectCheckType());
        collectDriverTransportIntention.setCollectType(param.getCollectType());
        collectDriverTransportIntention.setCreateUserNo(TokenUtil.getLoginUserInfo().getUserNo());
        collectDriverTransportIntention.setCreateUserName(TokenUtil.getLoginUserInfo().getUserName());
        // 逻辑删除状态，默认未删除
        collectDriverTransportIntention.setDeleteStatus(CollectDriverTransportIntentionEnum
                .DeleteStatusType.DELETE_STATUS_NO.getCode());
        // 状态，默认启用
        collectDriverTransportIntention.setStatus(CollectDriverTransportIntentionEnum
                .StatusType.STATUS_ENABLE.getCode());
        collectDriverTransportIntention.setCreateTime(LocalDateTime.now());
        collectDriverTransportIntention.setModifiedTime(LocalDateTime.now());
        // 1.保存信息主表
        collectDriverTransportIntentionDao.saveEntity(collectDriverTransportIntention);

        log.info("saveCarrierDriverCollectTransportIntention id:{}", collectDriverTransportIntention.getId());
        Integer id = collectDriverTransportIntention.getId();

        List<CollectDriverTransportIntentionRecord> recordList = new ArrayList<>();
        for (CarrierDriverCollectTransportIntentionRecordParam recordParam : param.getCollectIntentionRecordList()) {
            CollectDriverTransportIntentionRecord record = new CollectDriverTransportIntentionRecord();
            record.setCollectDriverTransportIntentionId(id);
            record.setCollectRecordSort(CollectDriverTransportIntentionEnum.DefaultSortType.DEFAULT_SORT.getCode());
            record.setCollectRecordName(recordParam.getCollectRecordName());
            record.setCreateUserNo(TokenUtil.getLoginUserInfo().getUserNo());
            record.setCreateUserName(TokenUtil.getLoginUserInfo().getUserName());
            record.setDeleteStatus(CollectDriverTransportIntentionEnum
                    .DeleteStatusType.DELETE_STATUS_NO.getCode());
            record.setCreateTime(LocalDateTime.now());
            record.setModifiedTime(LocalDateTime.now());
            recordList.add(record);
        }
        // 2.保存子表选项信息子表
        boolean result = collectDriverTransportIntentionRecordDao.saveBatchList(recordList);
        log.info("saveCarrierDriverCollectTransportIntention result:{}", result);

        return result ? 1 : 0;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateCarrierDriverCollectTransportIntention(CarrierDriverCollectTransportIntentionParam param) {
        if (Objects.isNull(param) || param.getCollectIntentionRecordList().isEmpty()) {
            // Todo 提示信息
            return 0;
        }
        // 1.修改主表信息
        CollectDriverTransportIntention updateCollectDriverTransportIntention = new CollectDriverTransportIntention();
        updateCollectDriverTransportIntention.setCollectName(param.getCollectName());
        updateCollectDriverTransportIntention.setCollectDescribe(param.getCollectDescribe());
        updateCollectDriverTransportIntention.setCollectCheckType(param.getCollectCheckType());
        updateCollectDriverTransportIntention.setModifiedTime(LocalDateTime.now());
        collectDriverTransportIntentionDao.updateEntityByKey(updateCollectDriverTransportIntention);
        // 2.删除子表信息
        collectDriverTransportIntentionRecordDao
                .deleteInField(CollectDriverTransportIntentionRecord::getCollectDriverTransportIntentionId,
                        Collections.singletonList(param.getId()));
        // 3.保存子表信息
        List<CollectDriverTransportIntentionRecord> recordList = new ArrayList<>();
        for (CarrierDriverCollectTransportIntentionRecordParam recordParam : param.getCollectIntentionRecordList()) {
            CollectDriverTransportIntentionRecord record = new CollectDriverTransportIntentionRecord();
            record.setCollectDriverTransportIntentionId(param.getId());
            record.setCollectRecordSort(CollectDriverTransportIntentionEnum.DefaultSortType.DEFAULT_SORT.getCode());
            record.setCollectRecordName(recordParam.getCollectRecordName());
            record.setCreateUserNo(TokenUtil.getLoginUserInfo().getUserNo());
            record.setCreateUserName(TokenUtil.getLoginUserInfo().getUserName());
            record.setDeleteStatus(CollectDriverTransportIntentionEnum
                    .DeleteStatusType.DELETE_STATUS_NO.getCode());
            record.setCreateTime(LocalDateTime.now());
            record.setModifiedTime(LocalDateTime.now());
            recordList.add(record);
        }
        // 2.保存子表选项信息子表
        boolean result = collectDriverTransportIntentionRecordDao.saveBatchList(recordList);
        log.info("updateCarrierDriverCollectTransportIntention result:{}", result);

        return result ? 1 : 0;
    }

    @Override
    public Integer enableCarrierDriverCollectTransportIntention(CarrierDriverCollectTransportIntentionParam param) {
        CollectDriverTransportIntention updateCollectDriverTransportIntention = new CollectDriverTransportIntention();
        updateCollectDriverTransportIntention.setId(param.getId());
        updateCollectDriverTransportIntention.setStatus(param.getStatus());
        return collectDriverTransportIntentionDao.updateEntityByKey(updateCollectDriverTransportIntention) ? 1 : 0;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer deleteCarrierDriverCollectTransportIntention(List<Integer> param) {
        // 1.删除主表信息
        collectDriverTransportIntentionDao.deleteInField(CollectDriverTransportIntention::getId, param);
        // 2.删除子表信息
        boolean result = collectDriverTransportIntentionRecordDao.deleteInField(CollectDriverTransportIntentionRecord::getCollectDriverTransportIntentionId, param);
        return result ? 1 : 0;
    }

    @Override
    public IPage<CarrierDriverCollectTransportIntentionVO> listCarrierDriverCollectTransportIntention(CarrierDriverCollectTransportIntentionListParam param) {
        IPage<CollectDriverTransportIntention> page = collectDriverTransportIntentionDao.listByCondition(param);
        List<CarrierDriverCollectTransportIntentionVO> list= collectDriverTransportIntentionDaoStruct.covertList(page.getRecords());
        return new Page<CarrierDriverCollectTransportIntentionVO>().setPages(page.getPages()).setTotal(page.getTotal()).setRecords(list);
    }

    @Override
    public List<CarrierDriverCollectTransportIntentionRecordVO> detailCarrierDriverCollectTransportIntention(Integer param) {
        List<CollectDriverTransportIntentionRecord> list = collectDriverTransportIntentionRecordDao
                .listByField(CollectDriverTransportIntentionRecord::getCollectDriverTransportIntentionId, param);
        List<CarrierDriverCollectTransportIntentionRecordVO> listVO = collectDriverTransportIntentionRecordDaoStruct.covertList(list);
        return listVO;
    }

    @Override
    public IPage<CarrierDriverCollectTransportIntentionVO> listCarrierDriverCollectTransportIntentionAll(AppCarrierDriverCollectTransportIntentionParam param) {
        // 1.查询所有已启用的信息
        IPage<CollectDriverTransportIntention> page = collectDriverTransportIntentionDao.listEnableInfo(param);
        if (CollectionUtils.isEmpty(page.getRecords())) {
            // todo 错误提示
            return null;
        }
        List<CarrierDriverCollectTransportIntentionVO> listVO = collectDriverTransportIntentionDaoStruct.covertList(page.getRecords());
        // 筛选主键id
        List<Integer> ids = listVO.stream().map(CarrierDriverCollectTransportIntentionVO::getId).toList();
        // 2.关联子表
        List<CollectDriverTransportIntentionRecord> recordList = collectDriverTransportIntentionRecordDao
                .listInField(CollectDriverTransportIntentionRecord::getCollectDriverTransportIntentionId, ids);
        if (CollectionUtils.isEmpty(recordList)) {
            // todo 错误提示
            return null;
        }

        List<CarrierDriverCollectTransportIntentionRecordVO> recordVOList
                = collectDriverTransportIntentionRecordDaoStruct.covertList(recordList);

        // 构建map
        Map<Integer, List<CarrierDriverCollectTransportIntentionRecordVO>> recordVoMap =
                recordVOList.stream().collect(Collectors
                        .groupingBy(CarrierDriverCollectTransportIntentionRecordVO :: getCollectDriverTransportIntentionId));
        // 遍历主编数据，构建子表数据
        for(CarrierDriverCollectTransportIntentionVO vo : listVO) {
            if (recordVoMap.containsKey(vo.getId())) {
                vo.setCollectIntentionRecordList(recordVoMap.get(vo.getId()));
            }
        }
        return new Page<CarrierDriverCollectTransportIntentionVO>().setPages(page.getPages()).setTotal(page.getTotal()).setRecords(listVO);
    }
}
